@@ -20,6 +20,7 @@ class InterfaceServer(BaseServer):
             aws_access_key_id, 
             aws_secret_access_key, 
             aws_sdb_reservation_domain, 
+            aws_s3_reservation_cache_bucket=None,
             aws_s3_http_cache_bucket=None,
             aws_s3_storage_bucket=None,
             aws_sdb_coordination_domain=None,
@@ -44,6 +45,7 @@ class InterfaceServer(BaseServer):
             self,
             aws_access_key_id, 
             aws_secret_access_key, 
+            aws_s3_reservation_cache_bucket=aws_s3_reservation_cache_bucket,
             aws_s3_http_cache_bucket=aws_s3_http_cache_bucket, 
             aws_sdb_reservation_domain=aws_sdb_reservation_domain, 
             aws_s3_storage_bucket=aws_s3_storage_bucket,
